@@ -22,9 +22,9 @@ pub trait SequenceEncoder {
 
     /// Push an element into the sequence.
     #[inline]
-    fn push<T>(&mut self, value: T) -> Result<(), Self::Error>
+    fn push<Mode, T>(&mut self, value: T) -> Result<(), Self::Error>
     where
-        T: Encode,
+        T: Encode<Mode>,
     {
         let encoder = self.next()?;
         value.encode(encoder)?;
@@ -48,11 +48,11 @@ pub trait PairsEncoder {
 
     /// Insert a pair immediately.
     #[inline]
-    fn insert<F, S>(&mut self, first: F, second: S) -> Result<(), Self::Error>
+    fn insert<Mode, F, S>(&mut self, first: F, second: S) -> Result<(), Self::Error>
     where
         Self: Sized,
-        F: Encode,
-        S: Encode,
+        F: Encode<Mode>,
+        S: Encode<Mode>,
     {
         self.next()?.insert(first, second)?;
         Ok(())
@@ -84,11 +84,11 @@ pub trait PairEncoder {
 
     /// Insert the pair immediately.
     #[inline]
-    fn insert<F, S>(mut self, first: F, second: S) -> Result<Self::Ok, Self::Error>
+    fn insert<Mode, F, S>(mut self, first: F, second: S) -> Result<Self::Ok, Self::Error>
     where
         Self: Sized,
-        F: Encode,
-        S: Encode,
+        F: Encode<Mode>,
+        S: Encode<Mode>,
     {
         self.first().and_then(|e| first.encode(e))?;
         self.second().and_then(|e| second.encode(e))?;
@@ -116,25 +116,25 @@ pub trait Encoder: Sized {
     /// The error raised by an encoder.
     type Error: Error;
     /// A simple pack that packs a sequence of elements.
-    type Pack<'this>: SequenceEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type Pack: SequenceEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// Encoder returned when encoding an optional value which is present.
-    type Some<'this>: Encoder<Ok = Self::Ok, Error = Self::Error>;
+    type Some: Encoder<Ok = Self::Ok, Error = Self::Error>;
     /// The type of a sequence encoder.
-    type Sequence<'this>: SequenceEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type Sequence: SequenceEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// The type of a tuple encoder.
-    type Tuple<'this>: SequenceEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type Tuple: SequenceEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// The type of a map encoder.
-    type Map<'this>: PairsEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type Map: PairsEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// Encoder that can encode a struct.
-    type Struct<'this>: PairsEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type Struct: PairsEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// Encoder that can encode a tuple struct.
-    type TupleStruct<'this>: PairsEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type TupleStruct: PairsEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// Encoder for a struct variant.
-    type StructVariant<'this>: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type StructVariant: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// Encoder for a tuple variant.
-    type TupleVariant<'this>: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type TupleVariant: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
     /// Encoder for a unit variant.
-    type UnitVariant<'this>: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
+    type UnitVariant: PairEncoder<Ok = Self::Ok, Error = Self::Error>;
 
     /// An expectation error. Every other implementation defers to this to
     /// report that something unexpected happened.
@@ -149,7 +149,7 @@ pub trait Encoder: Sized {
     ///
     /// struct EmptyStruct;
     ///
-    /// impl Encode for EmptyStruct {
+    /// impl<Mode> Encode<Mode> for EmptyStruct {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -162,7 +162,7 @@ pub trait Encoder: Sized {
     fn encode_unit(self) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Unit,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -183,27 +183,23 @@ pub trait Encoder: Sized {
     ///     data: [u8; 364],
     /// }
     ///
-    /// impl Encode for PackedStruct {
+    /// impl<Mode> Encode<Mode> for PackedStruct {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
-    ///         encoder.encode_pack(|mut pack| {
-    ///             pack.next()?.encode_u32(self.field)?;
-    ///             pack.next()?.encode_array(self.data)?;
-    ///             pack.end()
-    ///         })
+    ///         let mut pack = encoder.encode_pack()?;
+    ///         pack.next()?.encode_u32(self.field)?;
+    ///         pack.next()?.encode_array(self.data)?;
+    ///         pack.end()
     ///     }
     /// }
     /// ```
     #[inline]
-    fn encode_pack<T>(self, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::Pack<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_pack(self) -> Result<Self::Pack, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Pack,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -218,7 +214,7 @@ pub trait Encoder: Sized {
     ///     data: [u8; 364],
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -231,7 +227,7 @@ pub trait Encoder: Sized {
     fn encode_array<const N: usize>(self, _: [u8; N]) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Array,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -246,7 +242,7 @@ pub trait Encoder: Sized {
     ///     data: Vec<u8>,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -259,7 +255,7 @@ pub trait Encoder: Sized {
     fn encode_bytes(self, _: &[u8]) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Bytes,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -281,7 +277,7 @@ pub trait Encoder: Sized {
     ///     data: VecDeque<u8>,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -295,7 +291,7 @@ pub trait Encoder: Sized {
     fn encode_bytes_vectored(self, _: &[&[u8]]) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Bytes,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -310,7 +306,7 @@ pub trait Encoder: Sized {
     ///     data: String,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -323,7 +319,7 @@ pub trait Encoder: Sized {
     fn encode_string(self, _: &str) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::String,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -338,7 +334,7 @@ pub trait Encoder: Sized {
     ///     data: bool,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -351,7 +347,7 @@ pub trait Encoder: Sized {
     fn encode_bool(self, _: bool) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Bool,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -366,7 +362,7 @@ pub trait Encoder: Sized {
     ///     data: char,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -379,7 +375,7 @@ pub trait Encoder: Sized {
     fn encode_char(self, _: char) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Char,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -394,7 +390,7 @@ pub trait Encoder: Sized {
     ///     data: u8,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -407,7 +403,7 @@ pub trait Encoder: Sized {
     fn encode_u8(self, _: u8) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Unsigned8,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -422,7 +418,7 @@ pub trait Encoder: Sized {
     ///     data: u16,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -435,7 +431,7 @@ pub trait Encoder: Sized {
     fn encode_u16(self, _: u16) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Unsigned16,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -450,7 +446,7 @@ pub trait Encoder: Sized {
     ///     data: u32,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -463,7 +459,7 @@ pub trait Encoder: Sized {
     fn encode_u32(self, _: u32) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Unsigned32,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -478,7 +474,7 @@ pub trait Encoder: Sized {
     ///     data: u64,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -491,7 +487,7 @@ pub trait Encoder: Sized {
     fn encode_u64(self, _: u64) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Unsigned64,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -506,7 +502,7 @@ pub trait Encoder: Sized {
     ///     data: u128,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -519,7 +515,7 @@ pub trait Encoder: Sized {
     fn encode_u128(self, _: u128) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Unsigned128,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -534,7 +530,7 @@ pub trait Encoder: Sized {
     ///     data: i8,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -547,7 +543,7 @@ pub trait Encoder: Sized {
     fn encode_i8(self, _: i8) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Signed8,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -562,7 +558,7 @@ pub trait Encoder: Sized {
     ///     data: i16,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -575,7 +571,7 @@ pub trait Encoder: Sized {
     fn encode_i16(self, _: i16) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Signed16,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -590,7 +586,7 @@ pub trait Encoder: Sized {
     ///     data: i32,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -603,7 +599,7 @@ pub trait Encoder: Sized {
     fn encode_i32(self, _: i32) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Signed32,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -618,7 +614,7 @@ pub trait Encoder: Sized {
     ///     data: i64,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -631,7 +627,7 @@ pub trait Encoder: Sized {
     fn encode_i64(self, _: i64) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Signed64,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -646,7 +642,7 @@ pub trait Encoder: Sized {
     ///     data: i128,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -659,7 +655,7 @@ pub trait Encoder: Sized {
     fn encode_i128(self, _: i128) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Signed128,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -674,7 +670,7 @@ pub trait Encoder: Sized {
     ///     data: usize,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -687,7 +683,7 @@ pub trait Encoder: Sized {
     fn encode_usize(self, _: usize) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Usize,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -702,7 +698,7 @@ pub trait Encoder: Sized {
     ///     data: isize,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -715,7 +711,7 @@ pub trait Encoder: Sized {
     fn encode_isize(self, _: isize) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Isize,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -730,7 +726,7 @@ pub trait Encoder: Sized {
     ///     data: f32,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -743,7 +739,7 @@ pub trait Encoder: Sized {
     fn encode_f32(self, _: f32) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Float32,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -758,7 +754,7 @@ pub trait Encoder: Sized {
     ///     data: f64,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -771,7 +767,7 @@ pub trait Encoder: Sized {
     fn encode_f64(self, _: f64) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Float64,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -786,14 +782,14 @@ pub trait Encoder: Sized {
     ///     data: Option<String>,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
     ///         match &self.data {
     ///             Some(data) => {
-    ///                 encoder.encode_some(|encoder| data.encode(encoder))
+    ///                 encoder.encode_some().and_then(|e| Encode::<Mode>::encode(data, e))
     ///             }
     ///             None => {
     ///                 encoder.encode_none()
@@ -803,13 +799,10 @@ pub trait Encoder: Sized {
     /// }
     /// ```
     #[inline]
-    fn encode_some<T>(self, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::Some<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_some(self) -> Result<Self::Some, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Option,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -824,14 +817,14 @@ pub trait Encoder: Sized {
     ///     data: Option<String>,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
     ///         match &self.data {
     ///             Some(data) => {
-    ///                 encoder.encode_some(|encoder| data.encode(encoder))
+    ///                 encoder.encode_some().and_then(|e| Encode::<Mode>::encode(data, e))
     ///             }
     ///             None => {
     ///                 encoder.encode_none()
@@ -844,7 +837,7 @@ pub trait Encoder: Sized {
     fn encode_none(self) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Option,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -859,29 +852,26 @@ pub trait Encoder: Sized {
     ///     data: Vec<String>,
     /// }
     ///
-    /// impl Encode for MyType {
+    /// impl<Mode> Encode<Mode> for MyType {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
-    ///         encoder.encode_sequence(self.data.len(), |mut seq| {
-    ///             for element in &self.data {
-    ///                 seq.push(element)?;
-    ///             }
+    ///         let mut seq = encoder.encode_sequence(self.data.len())?;
     ///
-    ///             seq.end()
-    ///         })
+    ///         for element in &self.data {
+    ///             seq.push::<Mode, _>(element)?;
+    ///         }
+    ///
+    ///         seq.end()
     ///     }
     /// }
     /// ```
     #[inline]
-    fn encode_sequence<T>(self, _: usize, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::Sequence<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_sequence(self, _: usize) -> Result<Self::Sequence, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Sequence,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -894,39 +884,32 @@ pub trait Encoder: Sized {
     ///
     /// struct PackedTuple(u32, [u8; 364]);
     ///
-    /// impl Encode for PackedTuple {
+    /// impl<Mode> Encode<Mode> for PackedTuple {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
-    ///         encoder.encode_tuple(2, |mut tuple| {
-    ///             tuple.next()?.encode_u32(self.0)?;
-    ///             tuple.next()?.encode_array(self.1)?;
-    ///             tuple.end()
-    ///         })
+    ///         let mut tuple = encoder.encode_tuple(2)?;
+    ///         tuple.next()?.encode_u32(self.0)?;
+    ///         tuple.next()?.encode_array(self.1)?;
+    ///         tuple.end()
     ///     }
     /// }
     /// ```
     #[inline]
-    fn encode_tuple<T>(self, _: usize, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::Tuple<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_tuple(self, _: usize) -> Result<Self::Tuple, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Tuple,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
     /// Encode a map with a known length.
     #[inline]
-    fn encode_map<T>(self, _: usize, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::Map<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_map(self, _: usize) -> Result<Self::Map, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Map,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -942,27 +925,23 @@ pub trait Encoder: Sized {
     ///     age: u32,
     /// }
     ///
-    /// impl Encode for TupleStruct {
+    /// impl<Mode> Encode<Mode> for TupleStruct {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
-    ///         encoder.encode_struct(2, |mut st | {
-    ///             st.insert("name", &self.name)?;
-    ///             st.insert("age", self.age)?;
-    ///             st.end()
-    ///         })
+    ///         let mut st = encoder.encode_struct(2)?;
+    ///         st.insert::<Mode, _, _>("name", &self.name)?;
+    ///         st.insert::<Mode, _, _>("age", self.age)?;
+    ///         st.end()
     ///     }
     /// }
     /// ```
     #[inline]
-    fn encode_struct<T>(self, _: usize, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::Struct<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_struct(self, _: usize) -> Result<Self::Struct, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::Struct,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -975,26 +954,22 @@ pub trait Encoder: Sized {
     ///
     /// struct TupleStruct(String);
     ///
-    /// impl Encode for TupleStruct {
+    /// impl<Mode> Encode<Mode> for TupleStruct {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
-    ///         encoder.encode_tuple_struct(1, |mut tuple| {
-    ///             tuple.insert(0usize, &self.0)?;
-    ///             tuple.end()
-    ///         })
+    ///         let mut tuple = encoder.encode_tuple_struct(1)?;
+    ///         tuple.insert::<Mode, _, _>(0usize, &self.0)?;
+    ///         tuple.end()
     ///     }
     /// }
     /// ```
     #[inline]
-    fn encode_tuple_struct<T>(self, _: usize, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::TupleStruct<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_tuple_struct(self, _: usize) -> Result<Self::TupleStruct, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::TupleStruct,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -1007,7 +982,7 @@ pub trait Encoder: Sized {
     ///
     /// struct UnitStruct;
     ///
-    /// impl Encode for UnitStruct {
+    /// impl<Mode> Encode<Mode> for UnitStruct {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
@@ -1020,7 +995,7 @@ pub trait Encoder: Sized {
     fn encode_unit_struct(self) -> Result<Self::Ok, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::UnitStruct,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
@@ -1040,75 +1015,58 @@ pub trait Encoder: Sized {
     ///     }
     /// }
     ///
-    /// impl Encode for Enum {
+    /// impl<Mode> Encode<Mode> for Enum {
     ///     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     ///     where
     ///         E: Encoder
     ///     {
     ///         match self {
     ///             Enum::UnitVariant => {
-    ///                 encoder.encode_unit_variant(|mut variant| {
-    ///                     variant.first()?.encode_string("variant1")?;
-    ///                     variant.second()?.encode_unit()?;
-    ///                     variant.end()
-    ///                 })
+    ///                 let variant = encoder.encode_unit_variant()?;
+    ///                 variant.insert::<Mode, _, _>("variant1", ())
     ///             }
     ///             Enum::TupleVariant(data) => {
-    ///                 encoder.encode_tuple_variant(1, |mut variant| {
-    ///                     variant.first()?.encode_string("variant2")?;
-    ///                     variant.second()?.encode_string(data)?;
-    ///                     variant.end()
-    ///                 })
+    ///                 let variant = encoder.encode_tuple_variant(1)?;
+    ///                 variant.insert::<Mode, _, _>("variant2", data)
     ///             }
     ///             Enum::StructVariant { data, age } => {
-    ///                 encoder.encode_struct_variant(2, |mut variant| {
-    ///                     variant.first()?.encode_string("variant3")?;
+    ///                 let mut variant = encoder.encode_struct_variant(2)?;
+    ///                 variant.first()?.encode_string("variant3")?;
     ///
-    ///                     variant.second()?.encode_struct(2, |mut st| {
-    ///                         st.insert("data", data)?;
-    ///                         st.insert("age", age)?;
-    ///                         st.end()
-    ///                     })?;
+    ///                 let mut st = variant.second()?.encode_struct(2)?;
+    ///                 st.insert::<Mode, _, _>("data", data)?;
+    ///                 st.insert::<Mode, _, _>("age", age)?;
+    ///                 st.end()?;
     ///
-    ///                     variant.end()
-    ///                 })
+    ///                 variant.end()
     ///             }
     ///         }
     ///     }
     /// }
     /// ```
     #[inline]
-    fn encode_struct_variant<T>(self, _: usize, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::StructVariant<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_struct_variant(self, _: usize) -> Result<Self::StructVariant, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::StructVariant,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
     /// Encode an tuple enum variant.
     #[inline]
-    fn encode_tuple_variant<T>(self, _: usize, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::TupleVariant<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_tuple_variant(self, _: usize) -> Result<Self::TupleVariant, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::TupleVariant,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 
     /// Encode an unit enum variant.
     #[inline]
-    fn encode_unit_variant<T>(self, _: T) -> Result<Self::Ok, Self::Error>
-    where
-        T: FnOnce(Self::UnitVariant<'_>) -> Result<Self::Ok, Self::Error>,
-    {
+    fn encode_unit_variant(self) -> Result<Self::UnitVariant, Self::Error> {
         Err(Self::Error::message(InvalidType::new(
             expecting::UnitVariant,
-            &ExpectingWrapper(self),
+            &ExpectingWrapper::new(self),
         )))
     }
 }
@@ -1116,7 +1074,14 @@ pub trait Encoder: Sized {
 #[repr(transparent)]
 struct ExpectingWrapper<T>(T);
 
-impl<'de, T> Expecting for ExpectingWrapper<T>
+impl<T> ExpectingWrapper<T> {
+    #[inline]
+    const fn new(value: T) -> Self {
+        Self(value)
+    }
+}
+
+impl<T> Expecting for ExpectingWrapper<T>
 where
     T: Encoder,
 {
