@@ -18,7 +18,7 @@ use crate::de::{Decode, Decoder, PairDecoder, ValueVisitor};
 use crate::en::{Encode, Encoder, PairEncoder};
 use crate::error::Error;
 
-impl Encode for () {
+impl<Mode> Encode<Mode> for () {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -28,7 +28,7 @@ impl Encode for () {
     }
 }
 
-impl<'de> Decode<'de> for () {
+impl<'de, Mode> Decode<'de, Mode> for () {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -38,7 +38,7 @@ impl<'de> Decode<'de> for () {
     }
 }
 
-impl<T> Encode for marker::PhantomData<T> {
+impl<T, Mode> Encode<Mode> for marker::PhantomData<T> {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -48,7 +48,7 @@ impl<T> Encode for marker::PhantomData<T> {
     }
 }
 
-impl<'de, T> Decode<'de> for marker::PhantomData<T> {
+impl<'de, Mode, T> Decode<'de, Mode> for marker::PhantomData<T> {
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
@@ -60,12 +60,12 @@ impl<'de, T> Decode<'de> for marker::PhantomData<T> {
 
 macro_rules! atomic_impl {
     ($ty:ty) => {
-        impl<'de> Decode<'de> for $ty {
+        impl<'de, Mode> Decode<'de, Mode> for $ty {
             fn decode<D>(decoder: D) -> Result<Self, D::Error>
             where
                 D: Decoder<'de>,
             {
-                Decode::decode(decoder).map(Self::new)
+                Decode::<Mode>::decode(decoder).map(Self::new)
             }
         }
     };
@@ -85,22 +85,22 @@ atomic_impl!(AtomicUsize);
 
 macro_rules! non_zero {
     ($ty:ty) => {
-        impl Encode for $ty {
+        impl<Mode> Encode<Mode> for $ty {
             #[inline]
             fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
             where
                 E: Encoder,
             {
-                Encode::encode(&self.get(), encoder)
+                Encode::<Mode>::encode(&self.get(), encoder)
             }
         }
 
-        impl<'de> Decode<'de> for $ty {
+        impl<'de, Mode> Decode<'de, Mode> for $ty {
             fn decode<D>(decoder: D) -> Result<Self, D::Error>
             where
                 D: Decoder<'de>,
             {
-                let value = Decode::decode(decoder)?;
+                let value = Decode::<Mode>::decode(decoder)?;
 
                 match Self::new(value) {
                     Some(value) => Ok(value),
@@ -145,7 +145,7 @@ where
     }
 }
 
-impl<const N: usize> Encode for [u8; N] {
+impl<Mode, const N: usize> Encode<Mode> for [u8; N] {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -155,7 +155,7 @@ impl<const N: usize> Encode for [u8; N] {
     }
 }
 
-impl<'de, const N: usize> Decode<'de> for [u8; N] {
+impl<'de, Mode, const N: usize> Decode<'de, Mode> for [u8; N] {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -167,7 +167,7 @@ impl<'de, const N: usize> Decode<'de> for [u8; N] {
 
 macro_rules! impl_number {
     ($ty:ty, $read:ident, $write:ident) => {
-        impl Encode for $ty {
+        impl<Mode> Encode<Mode> for $ty {
             #[inline]
             fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
             where
@@ -177,7 +177,7 @@ macro_rules! impl_number {
             }
         }
 
-        impl<'de> Decode<'de> for $ty {
+        impl<'de, Mode> Decode<'de, Mode> for $ty {
             #[inline]
             fn decode<D>(decoder: D) -> Result<Self, D::Error>
             where
@@ -189,7 +189,7 @@ macro_rules! impl_number {
     };
 }
 
-impl Encode for bool {
+impl<Mode> Encode<Mode> for bool {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -199,7 +199,7 @@ impl Encode for bool {
     }
 }
 
-impl<'de> Decode<'de> for bool {
+impl<'de, Mode> Decode<'de, Mode> for bool {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -209,7 +209,7 @@ impl<'de> Decode<'de> for bool {
     }
 }
 
-impl Encode for char {
+impl<Mode> Encode<Mode> for char {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -219,7 +219,7 @@ impl Encode for char {
     }
 }
 
-impl<'de> Decode<'de> for char {
+impl<'de, Mode> Decode<'de, Mode> for char {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -244,7 +244,7 @@ impl_number!(i128, decode_i128, encode_i128);
 impl_number!(f32, decode_f32, encode_f32);
 impl_number!(f64, decode_f64, encode_f64);
 
-impl Encode for str {
+impl<Mode> Encode<Mode> for str {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
@@ -254,7 +254,7 @@ impl Encode for str {
     }
 }
 
-impl<'de> Decode<'de> for &'de str {
+impl<'de, Mode> Decode<'de, Mode> for &'de str {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -285,7 +285,7 @@ impl<'de> Decode<'de> for &'de str {
     }
 }
 
-impl Encode for [u8] {
+impl<Mode> Encode<Mode> for [u8] {
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
@@ -294,7 +294,7 @@ impl Encode for [u8] {
     }
 }
 
-impl<'de> Decode<'de> for &'de [u8] {
+impl<'de, Mode> Decode<'de, Mode> for &'de [u8] {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
@@ -325,9 +325,9 @@ impl<'de> Decode<'de> for &'de [u8] {
     }
 }
 
-impl<T> Encode for Option<T>
+impl<T, Mode> Encode<Mode> for Option<T>
 where
-    T: Encode,
+    T: Encode<Mode>,
 {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
@@ -335,15 +335,17 @@ where
         E: Encoder,
     {
         match self {
-            Some(value) => encoder.encode_some(|encoder| value.encode(encoder)),
+            Some(value) => encoder
+                .encode_some()
+                .and_then(|encoder| value.encode(encoder)),
             None => encoder.encode_none(),
         }
     }
 }
 
-impl<'de, T> Decode<'de> for Option<T>
+impl<'de, Mode, T> Decode<'de, Mode> for Option<T>
 where
-    T: Decode<'de>,
+    T: Decode<'de, Mode>,
 {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
@@ -358,56 +360,54 @@ where
     }
 }
 
-impl<T, U> Encode for Result<T, U>
+impl<T, U, Mode> Encode<Mode> for Result<T, U>
 where
-    T: Encode,
-    U: Encode,
+    T: Encode<Mode>,
+    U: Encode<Mode>,
 {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
     where
         E: Encoder,
     {
-        encoder.encode_struct_variant(1, |mut variant| {
-            match self {
-                Ok(ok) => {
-                    Encode::encode(&0usize, variant.first()?)?;
-                    Encode::encode(ok, variant.second()?)?;
-                }
-                Err(err) => {
-                    Encode::encode(&1usize, variant.first()?)?;
-                    Encode::encode(err, variant.second()?)?;
-                }
-            }
+        let mut variant = encoder.encode_struct_variant(1)?;
 
-            variant.end()
-        })
+        match self {
+            Ok(ok) => {
+                Encode::<Mode>::encode(&0usize, variant.first()?)?;
+                Encode::<Mode>::encode(ok, variant.second()?)
+            }
+            Err(err) => {
+                Encode::<Mode>::encode(&1usize, variant.first()?)?;
+                Encode::<Mode>::encode(err, variant.second()?)
+            }
+        }
     }
 }
 
-impl<'de, T, U> Decode<'de> for Result<T, U>
+impl<'de, Mode, T, U> Decode<'de, Mode> for Result<T, U>
 where
-    T: Decode<'de>,
-    U: Decode<'de>,
+    T: Decode<'de, Mode>,
+    U: Decode<'de, Mode>,
 {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
-        decoder.decode_variant(
-            |mut variant| match variant.first().and_then(usize::decode)? {
-                0 => variant.second().and_then(T::decode).map(Ok),
-                1 => variant.second().and_then(U::decode).map(Err),
-                tag => Err(D::Error::invalid_variant_tag("Result", tag)),
-            },
-        )
+        let mut variant = decoder.decode_variant()?;
+
+        match variant.first().and_then(<usize as Decode<Mode>>::decode)? {
+            0 => variant.second().and_then(T::decode).map(Ok),
+            1 => variant.second().and_then(U::decode).map(Err),
+            tag => Err(D::Error::invalid_variant_tag("Result", tag)),
+        }
     }
 }
 
-impl<T> Encode for Wrapping<T>
+impl<T, Mode> Encode<Mode> for Wrapping<T>
 where
-    T: Encode,
+    T: Encode<Mode>,
 {
     #[inline]
     fn encode<E>(&self, encoder: E) -> Result<E::Ok, E::Error>
@@ -418,15 +418,15 @@ where
     }
 }
 
-impl<'de, T> Decode<'de> for Wrapping<T>
+impl<'de, Mode, T> Decode<'de, Mode> for Wrapping<T>
 where
-    T: Decode<'de>,
+    T: Decode<'de, Mode>,
 {
     #[inline]
     fn decode<D>(decoder: D) -> Result<Self, D::Error>
     where
         D: Decoder<'de>,
     {
-        Ok(Wrapping(Decode::decode(decoder)?))
+        Ok(Wrapping(Decode::<Mode>::decode(decoder)?))
     }
 }
